using System;
using MediaPortal.GUI.Library;

namespace MyClickMania
{
  enum BlockStatus
  {
    visible = 0,
    disappearing,
    invisible,
  }

  enum GameStatus
  {
    Stopped = 0,
    Running,
    Disapering,
    MoveDown,
    MoveLeft,
    Check,
    Lost,
    Won,
  }

  public class ClickMania
  {
    #region const, structs and member variables
    // const
    private const int m_nSizeX = 8;
    private const int m_nSizeY = 12;
    private const int m_MaxColor = 5;
    private const int m_MaxMoves = (int)((m_nSizeX * m_nSizeY) / 2) + 2;

    // structs
    public struct Block
    {
      public int color;
      public int status;
    };

    // member variables
    private int m_nScore = 0;
    private int m_nMoves = 0;
    private int m_nStatus = (int)GameStatus.Stopped;
    private Block[,] m_Grid;
    private Block[, ,] m_GridHistory;
    private int[] m_ScoreHistory;

    #endregion

    #region Properties
    public int MaxX { get { return m_nSizeX; } }
    public int MaxY { get { return m_nSizeY; } }
    public int MaxColor { get { return m_MaxColor; } }
    public int Score { get { return m_nScore; } set { m_nScore = value; } }
    public int Status { get { return m_nStatus; } set { m_nStatus = value; } }
    public string GridString
    {
        get
        {
            string strTmp = "";
            for (int y = 0; y < m_nSizeY; y++)
            {
                for (int x = 0; x < m_nSizeX; x++)
                {
                    strTmp += m_Grid[x, y].status;
                    strTmp += m_Grid[x, y].color;
                }
            }
            return strTmp;
        }
        set
        {
            if (value.Length != 0 )
            {
                int i = 0;
                for (int y = 0; y < m_nSizeY; y++)
                {
                    for (int x = 0; x < m_nSizeX; x++)
                    {
                        m_Grid[x, y].status = int.Parse(value.Substring(i, 1)) ;
                        m_Grid[x, y].color = int.Parse(value.Substring(i + 1, 1));
                        i += 2;
                    }
                }
            }
        }
    }
    #endregion

    #region Indexer
    public Block this[int x, int y]
    {
      get { return m_Grid[x, y]; }  // ToDo: check x,y
      set { m_Grid[x, y] = value; }  // ToDo: check x,y
    }
    #endregion

    #region Init & Start
    public ClickMania()
    {
      m_Grid = new Block[m_nSizeX, m_nSizeY];
      m_GridHistory = new Block[m_MaxMoves, m_nSizeX, m_nSizeY];
      m_ScoreHistory = new int[m_MaxMoves];
      m_nStatus = (int)GameStatus.Stopped;
    }


    // Generates the grid
    public void Start()
    {
      if (m_nStatus != (int)GameStatus.Won)
      {
            m_nScore = 0;
      }
      m_nMoves = 0;
      m_nStatus = (int)GameStatus.Stopped;
      Random r = new Random(unchecked((int)DateTime.Now.Ticks));

      for (int i = 0; i < m_nSizeX; i++)
      {
        for (int j = 0; j < m_nSizeY; j++)
        {
          m_Grid[i, j].color = r.Next(0, m_MaxColor);
          m_Grid[i, j].status = (int)BlockStatus.visible;
        }
      }

      StoreMove();
      m_nStatus = (int)GameStatus.Running;
    }

    // Loads a previously saved grid
    public void LoadState(string gridstring, int score, int status)
    {
        m_nScore = score;
        m_nStatus = (int)GameStatus.Stopped;
        GridString = gridstring;
        StoreMove();
        m_nStatus = status;
    }
    
    #endregion

    #region Check Clicked Block
    // clicked on x, y pos in Array 
    // return number of found pos with equal color
    public int Clicked(int x, int y)
    {
      if (m_Grid[x, y].status == (int)BlockStatus.disappearing)
      {
        return 0;
      }
      int col = m_Grid[x, y].color;

      int px, py, nx, ny;  // 
      px = (x == 0 ? 0 : x - 1); //prior x
      py = (y == 0 ? 0 : y - 1); //prior y
      nx = (x == m_nSizeX - 1 ? m_nSizeX - 1 : x + 1);  // next x
      ny = (y == m_nSizeY - 1 ? m_nSizeY - 1 : y + 1);  // next y

      // check if min one neighour has same color
      if (((m_Grid[px, y].color == col) && (px != x) && (m_Grid[px, y].status == (int)BlockStatus.visible))
        || ((m_Grid[nx, y].color == col) && (nx != x) && (m_Grid[nx, y].status == (int)BlockStatus.visible))
        || ((m_Grid[x, py].color == col) && (py != y) && (m_Grid[x, py].status == (int)BlockStatus.visible))
        || ((m_Grid[x, ny].color == col) && (ny != y) && (m_Grid[x, ny].status == (int)BlockStatus.visible)))
      {
        int found = CheckPos(x, y, col);
        if (found > 1)
        {
          m_nStatus = (int)GameStatus.Disapering;
        }
        return found;
      }
      return 1;
    }


    public int CheckPos(int x, int y, int col)
    {
      if (m_Grid[x, y].status == (int)BlockStatus.disappearing)
      {
        return 0;
      }

      m_Grid[x, y].status = (int)BlockStatus.disappearing;

      int px, py, nx, ny;  // 
      px = (x == 0 ? 0 : x - 1); //prior x
      py = (y == 0 ? 0 : y - 1); //prior y
      nx = (x == m_nSizeX - 1 ? m_nSizeX - 1 : x + 1);  // next x
      ny = (y == m_nSizeY - 1 ? m_nSizeY - 1 : y + 1);  // next y
      int ret = 1;

      if ((m_Grid[px, y].status == (int)BlockStatus.visible) && (m_Grid[px, y].color == col))
      {
        ret += CheckPos(px, y, col);
      }

      if ((m_Grid[nx, y].status == (int)BlockStatus.visible) && (m_Grid[nx, y].color == col))
      {
        ret += CheckPos(nx, y, col);
      }

      if ((m_Grid[x, py].status == (int)BlockStatus.visible) && (m_Grid[x, py].color == col))
      {
        ret += CheckPos(x, py, col);
      }

      if ((m_Grid[x, ny].status == (int)BlockStatus.visible) && (m_Grid[x, ny].color == col))
      {
        ret += CheckPos(x, ny, col);
      }

      return ret;
    }
    #endregion

    #region Move Blocks
    public bool MakeInvisible()
    {
      bool ret = false;
      for (int i = 0; i < m_nSizeX; i++)
      {
        for (int j = 0; j < m_nSizeY; j++)
        {
          if (m_Grid[i, j].status == (int)BlockStatus.disappearing)
          {
            m_Grid[i, j].status = (int)BlockStatus.invisible;
            ret = true;
          }
        }
      }
      m_nStatus = (int)GameStatus.MoveDown;
      return ret;
    }

    public bool MoveDown()
    {
      bool ret = false;
      for (int i = 0; i < m_nSizeX; i++)
      {
        for (int j = m_nSizeY - 1; j >= 0; j--)
        {
          // if current pos invisible, then ...
          if (m_Grid[i, j].status == (int)BlockStatus.invisible)
          {
            // ... search for the next visible pos
            for (int k = j - 1; k >= 0; k--)
            {
              if (m_Grid[i, k].status == (int)BlockStatus.visible)
              {
                Block temp = m_Grid[i, j];
                m_Grid[i, j] = m_Grid[i, k];
                m_Grid[i, k] = temp;
                ret = true;
                break;
              }
            }
          }
        }
      }

      m_nStatus = (int)GameStatus.MoveLeft;
      return ret;
    }

    public bool MoveLeft()
    {
      bool ret = false;
      for (int i = 0; i < m_nSizeX; i++)
      {
        // check if first block is invisible, then ...
        if (m_Grid[i, m_nSizeY - 1].status == (int)BlockStatus.invisible)
        {
          // ... search for the next visible 
          for (int k = i + 1; k < m_nSizeX; k++)
          {
            if (m_Grid[k, m_nSizeY - 1].status == (int)BlockStatus.visible)
            {
              // move the complete row
              ret = true;
              for (int j = 0; j < m_nSizeY; j++)
              {
                Block temp = m_Grid[i, j];
                m_Grid[i, j] = m_Grid[k, j];
                m_Grid[k, j] = temp;
              }
              break;
            }
          }
        }
      }

      StoreMove();
      m_nStatus = (int)GameStatus.Check;
      return ret;
    }
    #endregion

    #region Check Won or Lost
    public bool CheckWon()
    {
      if (m_Grid[0, m_nSizeY - 1].status == (int)BlockStatus.invisible)
      {
        m_nStatus = (int)GameStatus.Won;
        return true;
      }
      m_nStatus = (int)GameStatus.Running;
      return false;
    }

    public bool CheckLost()
    {
      bool MoveLeft = false;
      //search for a possible move
      for (int i = 0; i < m_nSizeX; i++)
      {
        for (int j = m_nSizeY - 1; j >= 0; j--)
        {
          if (m_Grid[i, j].status == (int)BlockStatus.visible)
          {
            // test if around the visible block a other with the same color exists
            int px, py, nx, ny, col;
            px = (i == 0 ? 0 : i - 1); //prior x
            py = (j == 0 ? 0 : j - 1); //prior y
            nx = (i == m_nSizeX - 1 ? m_nSizeX - 1 : i + 1);  // next x
            ny = (j == m_nSizeY - 1 ? m_nSizeY - 1 : j + 1);  // next y
            col = m_Grid[i, j].color;

            // check if min one neighour has same color
            if (((m_Grid[px, j].color == col) && (px != i) && (m_Grid[px, j].status == (int)BlockStatus.visible))
              || ((m_Grid[nx, j].color == col) && (nx != i) && (m_Grid[nx, j].status == (int)BlockStatus.visible))
              || ((m_Grid[i, py].color == col) && (py != j) && (m_Grid[i, py].status == (int)BlockStatus.visible))
              || ((m_Grid[i, ny].color == col) && (ny != j) && (m_Grid[i, ny].status == (int)BlockStatus.visible)))
            {
              MoveLeft = true;
            }
          }
          else
          {
            break;
          }
        }
      }


      if (MoveLeft == false)
      {
        m_nStatus = (int)GameStatus.Lost;
        return true;
      }
      m_nStatus = (int)GameStatus.Running;
      return false;
    }
    #endregion

    #region History
    public void StoreMove()
    {
      for (int i = 0; i < m_nSizeX; i++)
      {
        for (int j = 0; j < m_nSizeY; j++)
        {
          m_GridHistory[m_nMoves, i, j].color = m_Grid[i, j].color;
          m_GridHistory[m_nMoves, i, j].status = m_Grid[i, j].status;
        }
      }
      m_ScoreHistory[m_nMoves] = Score;
      if (m_nMoves < m_MaxMoves - 1)
      {
        m_nMoves++;
      }
    }

    public void TakeBackMove()
    {
      int back = 0;
      if (m_nMoves > 1)
      {
        m_nMoves--;
        back = m_nMoves - 1;
      }

      Score = m_ScoreHistory[back];
      for (int i = 0; i < m_nSizeX; i++)
      {
        for (int j = 0; j < m_nSizeY; j++)
        {
          m_Grid[i, j].color = m_GridHistory[back, i, j].color;
          m_Grid[i, j].status = m_GridHistory[back, i, j].status;
        }
      }
    }
    #endregion
  }
}
