using System.Windows.Forms;
using System.Xml.Serialization;
using System.Drawing;

using MediaPortal.Configuration;
using MediaPortal.GUI.Library;
using MediaPortal.Dialogs;
using MediaPortal.Util;
using MediaPortal.Drawing;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Direct3D=Microsoft.DirectX.Direct3D;

namespace MyClickMania
{
  public class ClickManiaControl : GUIControl
  {
    #region Member variables

    ClickMania m_game = new ClickMania();	// the game itselfe without visualisation
    Settings m_Settings = new Settings();	// Serialization 
    //GUIFont		    m_Font;                         // fonts needed
    GUIImage[] m_imgBackgrounds;				// some backgrounds
    GUIImage[] m_imgBlocks;					// blocks in differnt colors
    GUIImage[] m_imgBlocksGlow;				// glowing blocks in different colors
    GUIImage[] m_imgCursors;                   // something for the remote control user
    int m_nxCursor = 0;			// x position of cursor
    int m_nyCursor = 0;			// y position of cursor
    int m_ncCount = 0;           // Counter for the cursor image   
    int m_nCursorTick = 0;           // cursor animation delay implementation
    int m_nCursorDelay = 25;			// the delay itselfe 
    int m_nxScale = 1;			// x scaling factor 
    int m_nyScale = 1;			// y scaling factor
    int m_nxOffset = 0;			// x offset of the board
    int m_nyOffset = 0;			// y offset of the board
    int m_nBoardWidth = 0;			// width of the board
    int m_nBoardHeight = 0;			// height of the board
    int m_nLastTick = 0;			// for delay implementation
    int m_nDelay = 150;			// the delay itselfe 
    System.DateTime m_dtStartTime;     // save the start time for the game timer  
    int m_dwPosX = 592;
    int m_dwPosY = 88;

    protected long m_dwColorDiffuse = 0xFFFFFFFF;	//

    #endregion

    #region Properties

    public int Score
    {
      get { return (m_game != null ? m_game.Score : 0); }
    }

    public int HighScore
    {
      get { return (m_Settings != null ? m_Settings.Highscore : 0); }
      set
      {
        if (m_Settings != null)
        {
          m_Settings.Highscore = value;
          m_Settings.Save();
        }
      }
    }

    public bool Sound
    {
      get { return (m_Settings != null ? m_Settings.Sound : false); }
      set
      {
        if (m_Settings != null)
        {
          m_Settings.Sound = value;
          m_Settings.Save();
        }
        if (value) GUIControl.SelectControl(GetID, 4);
        else GUIControl.DeSelectControl(GetID, 4);
      }
    }

    public bool KeyInterface
    {
      get { return (m_Settings != null ? m_Settings.KeyInterface : false); }
      set
      {
        if (m_Settings != null)
        {
          m_Settings.KeyInterface = value;
          m_Settings.Save();
        }
        if (value) GUIControl.SelectControl(GetID, 6);
        else GUIControl.DeSelectControl(GetID, 6);
      }
    }

    #endregion

    #region Serialization

    [System.Serializable]
    public class Settings
    {
      protected int m_nHighscore;
      protected bool m_bKeyInterface;
      protected bool m_bSound;
      protected string m_sGrid;
      protected int m_nScore;
      protected int m_nStatus;
      protected int m_nTicks;
      
      public Settings()
      {
        m_nHighscore = 0;
        m_bKeyInterface = false;
        m_bSound = false;
        m_sGrid = "";
        m_nScore = 0;
        m_nStatus = (int)GameStatus.Stopped;
        m_nTicks = 0;
      }

      [XmlElement("Highscore")]
      public int Highscore
      {
        get { return m_nHighscore; }
        set { m_nHighscore = value; }
      }

      [XmlElement("KeyInterface")]
      public bool KeyInterface
      {
        get { return m_bKeyInterface; }
        set { m_bKeyInterface = value; }
      }

      [XmlElement("Sound")]
      public bool Sound
      {
        get { return m_bSound; }
        set { m_bSound = value; }
      }

      [XmlElement("Grid")]
      public string Grid
      {
          get { return m_sGrid; }
          set { m_sGrid = value; }
      }

      [XmlElement("Score")]
      public int Score
      {
          get { return m_nScore; }
          set { m_nScore = value; }
      }

      [XmlElement("Status")]
      public int Status
      {
          get { return m_nStatus; }
          set { m_nStatus = value; }
      }

      [XmlElement("Ticks")]
      public int Ticks
      {
          get { return m_nTicks; }
          set { m_nTicks = value; }
      }

      public void Load()
      {
        using (MediaPortal.Profile.Settings xmlreader = new MediaPortal.Profile.Settings(Config.GetFile(Config.Dir.Config, "MyClickMania.xml")))
        {
          m_nHighscore = xmlreader.GetValueAsInt("clickmania", "highscore", 0);
          m_bKeyInterface = xmlreader.GetValueAsBool("clickmania", "keyinterface", false);
          m_bSound = xmlreader.GetValueAsBool("clickmania", "sound", true);
          m_sGrid = xmlreader.GetValueAsString("clickmania", "grid", "");
          m_nScore = xmlreader.GetValueAsInt("clickmania", "score", 0);
          m_nStatus = xmlreader.GetValueAsInt("clickmania", "status", (int)GameStatus.Stopped );
          m_nTicks = xmlreader.GetValueAsInt("clickmania", "ticks", 0);
        }
      }

      public void Save()
      {
        using (MediaPortal.Profile.Settings xmlwriter = new MediaPortal.Profile.Settings(Config.GetFile(Config.Dir.Config, "MyClickMania.xml")))
        {
          xmlwriter.SetValue("clickmania", "highscore", m_nHighscore);
          xmlwriter.SetValueAsBool("clickmania", "keyinterface", m_bKeyInterface);
          xmlwriter.SetValueAsBool("clickmania", "sound", m_bSound);
          xmlwriter.SetValue("clickmania", "grid", m_sGrid);
          xmlwriter.SetValue("clickmania", "score", m_nScore);
          xmlwriter.SetValue("clickmania", "status", m_nStatus);
          xmlwriter.SetValue("clickmania", "ticks", m_nTicks);
        }
      }
    }

    #endregion Serialization

    #region Init & Start Game
    public ClickManiaControl(int nParentID)
      : base(nParentID)
    {
      m_Settings.Load();

      GUIPropertyManager.SetProperty("#cm_status", " ");
      m_dtStartTime = System.DateTime.Now;  // store start time
      m_dtStartTime = m_dtStartTime.AddTicks(0 - (m_Settings.Ticks * 10000));
      m_nxCursor = 0;                // set cursor to the lower left side
      m_nyCursor = m_game.MaxY - 1;
      m_ncCount = 0;
      m_nCursorTick = System.Environment.TickCount;
      m_game.LoadState(m_Settings.Grid, m_Settings.Score, m_Settings.Status);
      GUIPropertyManager.SetProperty("#cm_score", m_game.Score.ToString());
    }


    public void Start(int m_dwControlID)
    {

      GUIControl.FocusControl(GetID, m_dwControlID);

      GUIPropertyManager.SetProperty("#cm_status", " ");
      m_dtStartTime = System.DateTime.Now;  // store start time
      m_nxCursor = 0;                // set cursor to the lower left side
      m_nyCursor = m_game.MaxY - 1;
      m_ncCount = 0;
      m_nCursorTick = System.Environment.TickCount;
      m_game.Start();
      GUIPropertyManager.SetProperty("#cm_score", m_game.Score.ToString());
      SaveGameState();
    }

    #endregion

    #region Overrides

    public override void FinalizeConstruction()
    {
      base.FinalizeConstruction();

      m_nBoardWidth = this.Width;
      m_nBoardHeight = this.Height;

      m_nyScale = m_nBoardHeight / (m_game.MaxY + 2);
      m_nxScale = m_nyScale;
      m_nBoardWidth = m_nxScale * m_game.MaxX;
      m_dwPosX = this._positionX;
      m_dwPosY = this._positionY;
      m_nxOffset = m_dwPosX + ((this.Width - (m_nxScale * m_game.MaxX)) / 2);
      m_nyOffset = m_dwPosY + ((this.Height - (m_nyScale * m_game.MaxY)) / 2);
      m_imgBlocks = new GUIImage[]
			{
				new GUIImage(GetID, 10001, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_red.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10002, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_blue.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10003, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_gray.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10004, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_yellow.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10005, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_cyan.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10006, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_orange.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10007, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_green.png", m_dwColorDiffuse),
			};

      m_imgBlocksGlow = new GUIImage[]
			{
				new GUIImage(GetID, 10011, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_red_glow.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10012, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_blue_glow.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10013, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_gray_glow.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10014, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_yellow_glow.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10015, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_cyan_glow.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10016, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_orange_glow.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10017, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\block_green_glow.png", m_dwColorDiffuse),
			};

      m_imgBackgrounds = new GUIImage[]
			{
				new GUIImage(GetID, 10021, m_dwPosX, m_dwPosY, this.Width, this.Height, GUIGraphicsContext.Skin + @"\media\clickmania\background.png", m_dwColorDiffuse),
				new GUIImage(GetID, 10022, m_dwPosX, m_dwPosY, this.Width, this.Height, GUIGraphicsContext.Skin + @"\media\clickmania\background_focus.png", m_dwColorDiffuse),
			};

      m_imgCursors = new GUIImage[]
			{
				new GUIImage(GetID, 10031, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\cursor1.png", Color.Black),
				new GUIImage(GetID, 10032, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\cursor2.png", Color.Black),
				new GUIImage(GetID, 10033, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\cursor3.png", Color.Black),
				new GUIImage(GetID, 10034, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\cursor4.png", Color.Black),
				new GUIImage(GetID, 10034, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\cursor5.png", Color.Black),
				new GUIImage(GetID, 10034, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\cursor6.png", Color.Black),
				new GUIImage(GetID, 10034, m_dwPosX, m_dwPosY, m_nxScale, m_nyScale, GUIGraphicsContext.Skin + @"\media\clickmania\cursor7.png", Color.Black),
			};
    }


    public override void AllocResources()
    {
      base.AllocResources();

      if (m_imgBlocks != null)
      {
        foreach (GUIImage image in m_imgBlocks)
        {
          image.AllocResources();
        }
      }


      if (m_imgBlocksGlow != null)
      {
        foreach (GUIImage image in m_imgBlocksGlow)
        {
          image.AllocResources();
        }
      }

      if (m_imgBackgrounds != null)
      {
        foreach (GUIImage image in m_imgBackgrounds)
        {
          image.AllocResources();
        }
      }

      if (m_imgCursors != null)
      {
        foreach (GUIImage image in m_imgCursors)
        {
          image.AllocResources();
        }
      }
    }

    public override void OnAction(Action action)
    {
      if (m_game.Status != (int)GameStatus.Stopped)
      {
        switch (action.wID)
        {
          case Action.ActionType.ACTION_SELECT_ITEM:
          case Action.ActionType.ACTION_KEY_PRESSED:
            if ((KeyInterface == true) && (m_game.Status == (int)GameStatus.Running) /* && (action.m_key.KeyCode == 0x13) */)
            {
              ProcessBlockClicked(m_nxCursor, m_nyCursor);
            }
            else
            {
              base.OnAction(action);
            }
            break;

          case Action.ActionType.ACTION_MOUSE_CLICK:
            if (m_game.Status == (int)GameStatus.Running)
            {
              // calc the block where the mouse was clicked on
              int x = (int)((int)action.fAmount1 - m_nxOffset) / m_nxScale;
              int y = (int)((int)action.fAmount2 - m_nyOffset) / m_nyScale;
              if ((x > -1) & (x < 8) & (y > -1) & (y < 12))
                ProcessBlockClicked(x, y);
            }
            else
            {
              base.OnAction(action);
            }
            break;

          case Action.ActionType.ACTION_MOVE_UP:
            if (m_nyCursor > 0) m_nyCursor -= 1;
            break;
          case Action.ActionType.ACTION_MOVE_DOWN:
            if (m_nyCursor < m_game.MaxY - 1) m_nyCursor += 1;
            break;
          case Action.ActionType.ACTION_MOVE_LEFT:
            if (m_nxCursor > 0) m_nxCursor -= 1;
            else
              base.OnAction(action);
            break;
          case Action.ActionType.ACTION_MOVE_RIGHT:
            if (m_nxCursor < m_game.MaxX - 1) m_nxCursor += 1;
            break;
          case Action.ActionType.ACTION_CONTEXT_MENU:
          case Action.ActionType.ACTION_SHOW_INFO:
            TakeBackMove();
            break;
          default:
            base.OnAction(action);
            break;
        }
      }
      else
      {
        base.OnAction(action);
      }
    }


    public override void Render(float timePassed)
    {
      RenderBackground(timePassed);

      if (m_game.Status != (int)GameStatus.Stopped)
      {
        // show timer only when game is running
        if (m_game.Status < (int)GameStatus.Lost)
        {
          System.TimeSpan ts = System.DateTime.Now.Subtract(m_dtStartTime);
          GUIPropertyManager.SetProperty("#cm_time", System.String.Format("{0:D2}:{1:D2}:{2:D2}", ts.Hours, ts.Minutes, ts.Seconds));
        }

        for (int i = 0; i < m_game.MaxX; i++)
        {
          int posX = m_nxOffset + (int)(i * m_nxScale);
          for (int j = 0; j < m_game.MaxY; j++)
          {
            int posY = m_nyOffset + (int)(j * m_nyScale);
            int col = m_game[i, j].color;

            switch (m_game[i, j].status)
            {
              case (int)BlockStatus.visible:
                if (m_imgBlocks != null && m_imgBlocks[col] != null)
                {
                  m_imgBlocks[col].SetPosition(posX, posY);
                  m_imgBlocks[col].Render(timePassed);
                }
                break;
              case (int)BlockStatus.disappearing:
                if (m_imgBlocks != null && m_imgBlocks[col] != null)
                {
                  m_imgBlocksGlow[col].SetPosition(posX, posY);
                  m_imgBlocksGlow[col].Render(timePassed);
                }
                break;
            }
          }
        }

        switch (m_game.Status)
        {
          case (int)GameStatus.Disapering:
            if ((System.Environment.TickCount - m_nLastTick) > m_nDelay)
            {
              if (m_game.MakeInvisible() == true)
              {
                //if (Sound == true) Utils.PlaySound("ClickMania.Block.wav", false, true);
              }
              m_nLastTick = System.Environment.TickCount;
            }
            break;

          case (int)GameStatus.MoveDown:
            if ((System.Environment.TickCount - m_nLastTick) > m_nDelay)
            {
              if (m_game.MoveDown() == true)
              {
                //if (Sound == true) Utils.PlaySound("ClickMania.MoveDown.wav", false, true);
              }
              m_nLastTick = System.Environment.TickCount;
            }
            break;

          case (int)GameStatus.MoveLeft:
            if ((System.Environment.TickCount - m_nLastTick) > m_nDelay)
            {
              if (m_game.MoveLeft() == true)
              {
                if (m_game.Status == (int)GameStatus.Running)
                {
                  if (Sound == true) Utils.PlaySound("ClickMania.MoveLeft.wav", false, true);
                }
              }
              m_nLastTick = System.Environment.TickCount;
            }
            break;
          case (int)GameStatus.Check:
            if (m_game.CheckWon() == true)  // did we won?
            {
              if (Sound == true) Utils.PlaySound("ClickMania.Won.wav", false, true);
            }
            else if (m_game.CheckLost() == true) // no more move possible?
            {
              GUIPropertyManager.SetProperty("#cm_status", "Game Over");
              if (Sound == true) Utils.PlaySound("ClickMania.GameOver.wav", false, true);
            }
            break;

          case (int)GameStatus.Won:
            GUIPropertyManager.SetProperty("#cm_status", System.String.Format("Your Score: {0}", m_game.Score));
            break;

          case (int)GameStatus.Lost:
            GUIPropertyManager.SetProperty("#cm_status", "Game Over");
            break;

        }
        RenderCursor(timePassed);
      }
    }

    public void RenderBackground(float timePassed)
    {
      if (IsFocused)
      {
        m_imgBackgrounds[1].Render(timePassed);
      }
      else
      {
        m_imgBackgrounds[0].Render(timePassed);
      }
    }

    public void RenderCursor(float timePassed)
    {
      if ((KeyInterface) && (m_imgCursors != null))
      {
        int posX = m_nxOffset + (int)(m_nxCursor * m_nxScale);  //can be optimized
        int posY = m_nyOffset + (int)(m_nyCursor * m_nyScale);  //can be optimized
        m_imgCursors[m_ncCount].SetPosition(posX, posY);
        m_imgCursors[m_ncCount].Render(timePassed);
        if ((System.Environment.TickCount - m_nCursorTick) > m_nCursorDelay)
        {
          m_nCursorTick = System.Environment.TickCount;
          m_ncCount += 1;
          if (m_ncCount >= m_imgCursors.Length)
          {
            m_ncCount = 0;
          }
        }
      }
    }


    public void ProcessBlockClicked(int x, int y)
    {
      int found = m_game.Clicked(x, y);
      if (found > 1)
      {
        if (Sound == true) Utils.PlaySound("ClickMania.Block.wav", false, true);
        m_game.Score += found * found;
        if (m_game.Score > m_Settings.Highscore)
        {
          m_Settings.Highscore = m_game.Score;
          m_Settings.Save();
        }
        GUIPropertyManager.SetProperty("#cm_score", m_game.Score.ToString());
        GUIPropertyManager.SetProperty("#cm_highscore", m_Settings.Highscore.ToString());
        m_nLastTick = System.Environment.TickCount;
        SaveGameState();
      }
    }

    #endregion

    public void TakeBackMove()
    {
      if (m_game.Status == (int)GameStatus.Running)
      {
        m_game.TakeBackMove();
        GUIPropertyManager.SetProperty("#cm_score", m_game.Score.ToString());
        SaveGameState();
      }
    }

    public void SaveGameState()
    {
        m_Settings.Grid = m_game.GridString;
        m_Settings.Score = m_game.Score;
        m_Settings.Status = m_game.Status;
        m_Settings.Ticks = (int)((System.DateTime.Now.Ticks - m_dtStartTime.Ticks) / 10000);
        m_Settings.Save();
    }
  }
}
