using System.Windows.Forms;

using MediaPortal.GUI.Library;
using MediaPortal.Dialogs;
using MediaPortal.Configuration;

namespace MyClickMania
{
  public class myClickMania : GUIWindow, ISetupForm
  {
    #region Member variables & enums
    ClickManiaControl m_wndClickMania;  // Link to our control

    enum Controls
    {
      New = 2,
      Sound = 4,
      KeyInterface = 6,
      TakeBack = 8,

      Score = 202,
      HighScore = 204,
    }

    string pluginTitle;
    public const string defaultPluginTitle = "ClickMania";
    
    public const string configFileName = "ClickMania.xml";
    public const string configSection = "clickmania";

    #endregion

    #region Constructor
    public myClickMania()
    {
      //
      // TODO: Add constructor logic here
      //
    }
    #endregion

    #region ISetupForm

    public string PluginName()
    {
      return defaultPluginTitle;
    }

    public string Description()
    {
      return "ClickMania for MediaPortal";
    }

    public string Author()
    {
      return "Bavarian";
    }

    // show the setup dialog
    public void ShowPlugin()
    {
      System.Windows.Forms.Form setup = new ClickManiaSetup();
      setup.ShowDialog();
    }

    public bool HasSetup()
    {
      return true;
    }

    public bool CanEnable()
    {
      return true;
    }

    public bool DefaultEnabled()
    {
      return true;
    }

    public int GetWindowId()
    {
      return GetID;
    }

    public bool GetHome(out string strButtonText, out string strButtonImage, out string strButtonImageFocus, out string strPictureImage)
    {
      /*
      using (MediaPortal.Profile.Settings xmlreader = new MediaPortal.Profile.Settings(Config.GetFile(Config.Dir.Config, myClickMania.configFileName)))
      {
        pluginTitle = xmlreader.GetValueAsString(myClickMania.configSection, "pluginName", defaultPluginTitle);
      }
      */

      strButtonText = pluginTitle;
      strButtonImage = System.String.Empty;
      strButtonImageFocus = System.String.Empty;
      strPictureImage = "hover_clickmania.png";
      return true;
    }

    #endregion

    #region Overriedes

    public override int GetID
    {
      get
      {
        return 5555;
      }
      set
      {
      }
    }

    public override bool Init()
    {
      // register our own GUIControl
      GUIControlFactory.RegisterControl("clickmania", typeof(ClickManiaControl));

      using (MediaPortal.Profile.Settings xmlreader = new MediaPortal.Profile.Settings(Config.GetFile(Config.Dir.Config, myClickMania.configFileName)))
      {
        pluginTitle = xmlreader.GetValueAsString(myClickMania.configSection, "pluginName", defaultPluginTitle);
      }

      return Load(GUIGraphicsContext.Skin + @"\myclickmania.xml");
    }

    protected override void OnClicked(int controlId, GUIControl control, Action.ActionType actionType)
    {
      switch (controlId)
      {
        case (int)Controls.New:
          {
            m_wndClickMania.Start(controlId);
            break;
          }

        case (int)Controls.TakeBack:
          {
            m_wndClickMania.TakeBackMove();
            break;
          }

        case (int)Controls.Sound:
          {
            m_wndClickMania.Sound = !m_wndClickMania.Sound;
            break;
          }
        case (int)Controls.KeyInterface:
          {
            m_wndClickMania.KeyInterface = !m_wndClickMania.KeyInterface;
            break;
          }
      }

      base.OnClicked(controlId, control, actionType);
    }

    public override bool OnMessage(GUIMessage message)
    {
      switch (message.Message)
      {
        case GUIMessage.MessageType.GUI_MSG_WINDOW_INIT:
          base.OnMessage(message);
          GUIPropertyManager.SetProperty("#currentmodule", pluginTitle);
          return OnInit();
      }

      return base.OnMessage(message);
    }

    #endregion

    #region Implementation

    public bool OnInit()
    {
      m_wndClickMania = GetControl(10) as ClickManiaControl;
      m_wndClickMania.RenderBackground(0);

      if (m_wndClickMania.Sound)
      {
        GUIControl.SelectControl(GetID, (int)Controls.Sound);
      }
      else
      {
        GUIControl.DeSelectControl(GetID, (int)Controls.Sound);
      }
      if (m_wndClickMania.KeyInterface)
      {
        GUIControl.SelectControl(GetID, (int)Controls.KeyInterface);
      }
      else
      {
        GUIControl.DeSelectControl(GetID, (int)Controls.KeyInterface);
      }

      GUIPropertyManager.SetProperty("#cm_score", m_wndClickMania.Score.ToString());
      GUIPropertyManager.SetProperty("#cm_highscore", m_wndClickMania.HighScore.ToString());
      GUIPropertyManager.SetProperty("#cm_time", " ");
      GUIPropertyManager.SetProperty("#cm_status", " ");
      GUIPropertyManager.SetProperty("#cm_title", pluginTitle);

      return true;
    }

    #endregion
  }
}
